% simuDiag : "simultaneous diagonalization" of two posdef symmetric matrices
%
% [T D]=simuDiag(Cn,Cx)
%
% Inputs: 
% Cn, Cx - positive definite symmetric input matrices
%
% Outputs :
% T,D
% T is dense, D is diagonal, such that T*T'=Cn and T*D*T'=Cx

function [T D]=simuDiag(Cn,Cx)
[U L K]=svd(Cn);
% throw away K. if matrix is full rank then U should equal K as it is
% symmetric, if it is not full rank then the columns where U <> K don't
% matter.
F=U*(L^.5);
iF=inv(F);
[V D K]=svd(iF*Cx*iF');
T=F*V;
if(~isreal(T) | ~isreal(D))
  error('imaginary matrices returned');
end
% now should have Cn=T*T' and Cx=T*D*T'
