% upsampleFourier : upsample by zero padding in Fourier domain
%
% r = upsampleFourier(x,ndim)
%
% This interpolates the image x up to dimensions ndim (which must be larger
% than the size of x) by zero-padding in the fourier domain , preserving
% average power. So sum(abs(r(:)^2))/prod(ndim) =
% sum(abs(x(:)^2))/prod(size(x)) after this function is called.
%
% This won't work quite correctly if x has even dimensions and has fourier
% transform that doesn't go to zero right at the nyquist limit - as the zero
% padded dft won't have odd imaginary part at the highest frequency, but it
% will work fine for even dimension pyramid bands as they are band limited.
%
% If x is real then r is set to be real
%
% Inputs: 
% x - input array
% ndim - desired size of output array
%
% Outputs:
% r- upsampled array (of size ndim)

function r=upsampleFourier(x,ndim)
dft=fftshift(fft2(x));
dft_big=up2(dft,ndim);
if(isreal(x))
  r=real(ifft2(ifftshift(dft_big)));
else
  r=ifft2(ifftshift(dft_big));
end
r=r*prod(ndim)/prod(size(x));

function r=up2(x,ndim)
dim=size(x);
ctr=ceil((dim+0.5)/2);
nctr=ceil((ndim+0.5)/2);
r=zeros(ndim);
start_p=nctr-ctr+1;
end_p=start_p+dim-1;
r(start_p(1):end_p(1),start_p(2):end_p(2))=x;
